/**************************************************************************************************
  I2C.c
  I2C1 driver.
  The I2C circuit in this microcontroller is mostly software driven. Using while loops to wait for
  I2C events hangs up the CPU and prevents other tasks from running concurrently. This I2C
  implementation uses a state machine to break up the software loops necessary in making the
  interface work properly.
  
  The state machine has only 2 sequences, one for writing and another for reading registers in
  the STMPR610 chip.
**************************************************************************************************/

// Include Files
#include "I2C.h"
#include "Buffers.h"
#include "USB_Driver.h"
#include "RGB.h"
#include "UART6.h"
#include "Timers.h"
#include "Common.h"



// Constants
const code char STMPE610ADDR = 0x88;


// Variables
char RdRegAddr = 0;
char RdByteCnt = 2;
char RdBuffer[BUFSIZ];
char *RdBufPtr;

char WrRegAddr = 0;
char WrByteCnt = 0;
char WrBuffer[BUFSIZ];
char *WrBufPtr;

unsigned short int I2C1Timer;
char I2C1_State = 0;         // Start in state 0

char PrintBuf[32];           // Temporary formatting buffer




/**************************************************************************************************
  Initialize hardware without using mikroE library.
**************************************************************************************************/
void I2C1_Initialize(void) {

     //I2C1_Init_Advanced(400000, &_GPIO_MODULE_I2C1_PB67);  // Use this to see how registers are set

     //GPIO_Alternate_Function_Enable(&_GPIO_MODULE_I2C1_PB67);

     // GPIOB
     GPIOBEN_bit = 1;                  // Set GPIOB clock enable bit

     GPIOB_MODER   |= 0x0000A000;      // Alternate function on bits 6 and 7
     GPIOB_OTYPER  |= 0x000000C0;      // Output open-drain on bits 6 and 7
     GPIOB_OSPEEDR |= 0x0000F000;      // Outpu high speed on bits 6 and 7
     GPIOB_PUPDR    = 0x00000000;      // No pull-up/pull-down on port B
     GPIOB_AFRL    |= 0x44000000;      // Select alternate function I2C1 for bits 6 and 7

     // I2C1
     I2C1EN_bit = 1;                   // Enable I2C1 peripheral clock

     SWRST_I2C1_CR1_bit = 1;           // Execute software reset
     delay_ms(10);
     SWRST_I2C1_CR1_bit = 0;

     I2C1_CR2 = 0x001E;                // Set peripheral clock register
     I2C1_CCR = 0xC003;                // Configure clock control register
     I2C1_TRISE = 0x000A;              // Configure rise time register
     PE_I2C1_CR1_bit = 1;              // Enable I2C1 module

     DEBUGOUT("I2C1 initialized\r\n");
}

/**************************************************************************************************
  The read registers function.
  Input:
     Addr: starting register address
     Cnt:  number of registers to read out (limit to 8 or less)
     *Bfr: pointer to the buffer where register data will be placed
**************************************************************************************************/
void ReadRegs(char Addr, char Cnt, char *Bfr) {
#if DEBUG
     char *BufNum;
#endif

     RdRegAddr = Addr;
     RdByteCnt = Cnt;
     RdBufPtr = Bfr;
#if DEBUG
     BufNum = GetBuffer();
     sprintf(BufNum,"Read address: %02X  Count: %02X  Buffer pointer: %08LX\r\n",
     RdRegAddr,RdByteCnt,RdBufPtr);
     UART6_PutQueue(BufNum);
#endif
     DEBUGOUT("Going to state 1\r\n");
     I2C1_State = 1;
}

/**************************************************************************************************
  The write registers function.
  Input:
     Addr: starting register address
     Cnt:  number of registers to write into (limit to 8 or less)
     *Bfr: pointer to the buffer where register data will come from
**************************************************************************************************/
void WriteRegs(char Addr, char Cnt, char *Bfr) {
     int i;
#if DEBUG
     char *BufNum;
#endif
     
     WrRegAddr = Addr;
     WrByteCnt = Cnt;
     WrBufPtr = Bfr;
#if DEBUG
     BufNum = GetBuffer();
     sprintf(BufNum,"Write address: %02X  Count: %02X  Buffer pointer: %08LX\r\n",
     WrRegAddr,WrByteCnt,WrBufPtr);
     UART6_PutQueue(BufNum);
#endif
     for (i=0;i<Cnt;i++)
          PrintBuf[i] = Bfr[i];
#if DEBUG
     BufNum = GetBuffer();
     sprintf(BufNum,"Write buffer: %02X %02X %02X %02X %02X %02X %02X %02X\r\n",PrintBuf[0],
     PrintBuf[1],PrintBuf[2],PrintBuf[3],PrintBuf[4],PrintBuf[5],PrintBuf[6],PrintBuf[7]);
     UART6_PutQueue(BufNum);
#endif
     DEBUGOUT("Going to state 18\r\n");
     I2C1_State = 18;
}

/**************************************************************************************************
  Diagnostic - display 8 bytes of read buffer.
**************************************************************************************************/
void DisplayData(char *Bfr, char Cnt) {
     int i;
#if DEBUG
     char *BufNum;
#endif

     memset(RdBuffer,0,BUFSIZ);
     if (Cnt > 8) {
         for (i=0;i<8;i++)
             RdBuffer[i] = Bfr[i];
     } else {
         for (i=0;i<Cnt;i++)
             RdBuffer[i] = Bfr[i];
     }

#if DEBUG
     // Diagnostic to display the address of the read buffer and the byte count
     BufNum = GetBuffer();
     sprintf(BufNum,"Buffer pointer: %08LX Count: %02X\r\n",Bfr,Cnt);
     UART6_PutQueue(BufNum);

     // Diagnostic to display the read buffer
     BufNum = GetBuffer();
     sprintf(BufNum,"Read buffer: %02X %02X %02X %02X %02X %02X %02X %02X\r\n",RdBuffer[0],
     RdBuffer[1],RdBuffer[2],RdBuffer[3],RdBuffer[4],RdBuffer[5],RdBuffer[6],RdBuffer[7]);
     UART6_PutQueue(BufNum);
#endif
}

/**************************************************************************************************
  The timeout subroutine.
**************************************************************************************************/
void TimeoutCheck(unsigned TimeoutState) {
     char *BufNum;

     if (check_timer(I2C1Timer) == 0) {          // If timeout

         BufNum = GetBuffer();
         sprintf(BufNum,"Timeout in state %d\r\n",TimeoutState);
         UART6_PutQueue(BufNum);
         free_timer(I2C1Timer);                  // Release Wi-Fi timer back to the pool
         I2C1_State = 0;                         // Return to holding state
     }
}

/**************************************************************************************************
  The I2C1 state machine
**************************************************************************************************/
void I2C1_SM(void) {
     static char *TempPtr, TempCnt;              // Must be static
#if DEBUG
     static char *BufNum;
#endif


     switch (I2C1_State) {                       // Hold here until user changes state
         case 0:
             break;

     /*********************************************************************************************
       Read registers sequence.
       Assumption: user has set up the starting register address - RdRegAddr
                   user has set up the number of registers used - RdByteCnt
                   user has set up a pointer to a buffer - RdBufPtr
     *********************************************************************************************/

         case 1:
             I2C1Timer = get_timer();            // Get a timer handle
             set_timer(I2C1Timer,500);           // Set timer
             I2C1_SR1 = 0x0000;

             // Save these variables for later use
             TempPtr = RdBufPtr;
             TempCnt = RdByteCnt;
/*
             // Diagnostic
             BufNum = GetBuffer();
             sprintf(BufNum,"%08LX\r\n",TempPtr);
             UART6_PutQueue(BufNum);
*/
             DEBUGOUT("Going to state 2\r\n");
             I2C1_State = 2;
             break;

         case 2:
             if (!BUSY_I2C1_SR2_bit) {           // Make sure bus is not busy
                 START_I2C1_CR1_bit = 1;         // Send Start bit
                 set_timer(I2C1Timer,500);
                 DEBUGOUT("Going to state 3\r\n");
                 I2C1_State = 3;
             }
             TimeoutCheck(2);
             break;

         case 3:
             if (SB_I2C1_SR1_bit) {              // When Start Flag is set
                 I2C1_DR = (STMPE610ADDR);       // Send slave address + write
                 set_timer(I2C1Timer,500);
                 DEBUGOUT("Going to state 4\r\n");
                 I2C1_State = 4;
             }
             TimeoutCheck(3);
             break;

         case 4:
             if (ADDR_I2C1_SR1_bit) {            // Read SR1 waiting for ADDR bit
                 set_timer(I2C1Timer,500);
                 DEBUGOUT("Going to state 5\r\n");
                 I2C1_State = 5;
             }
             TimeoutCheck(4);
             break;

         case 5:
             if (TRA_I2C1_SR2_bit) {             // Read SR2 - this will clear ADDR in SR1
                 DEBUGOUT("Going to state 6\r\n");
                 I2C1_State = 6;
             }
             TimeoutCheck(5);
             break;

         case 6:
             I2C1_DR = RdRegAddr;                // Send the register address that is to be read
             set_timer(I2C1Timer,500);
             DEBUGOUT("Going to state 7\r\n");
             I2C1_State = 7;
             break;

         case 7:
             if (TxE_I2C1_SR1_bit) {             // Wait for transmitter empty bit
                 DEBUGOUT("Going to state 8\r\n");
                 I2C1_State = 8;
             }
             TimeoutCheck(7);
             break;

         case 8:
             START_I2C1_CR1_bit = 1;             // Set ReStart bit
             set_timer(I2C1Timer,500);
             DEBUGOUT("Going to state 9\r\n");
             I2C1_State = 9;
             break;

         case 9:
             if (SB_I2C1_SR1_bit) {              // When Start Flag is set
                 I2C1_DR = (STMPE610ADDR|0x01);  // Send slave address + read
                 set_timer(I2C1Timer,500);
                 DEBUGOUT("Going to state 10\r\n");
                 I2C1_State = 10;
             }
             TimeoutCheck(9);
             break;

         case 10:
             if (ADDR_I2C1_SR1_bit) {            // Read SR1 waiting for ADDR bit
                 set_timer(I2C1Timer,500);
                 DEBUGOUT("Going to state 11\r\n");
                 I2C1_State = 11;
             }
             TimeoutCheck(10);
             break;

         case 11:
             if (!TRA_I2C1_SR2_bit) {            // Read SR2 - this will clear ADDR in SR1
                 
                 
                 RdBufPtr++;                     // Reserve buffer position 0 for valid flag
                 
                 
                 if (RdByteCnt > 1) {
                     DEBUGOUT("Going to state 12\r\n");
                     I2C1_State = 12;            // More than 1 byte to read
                 } else {
                     DEBUGOUT("Going to state 15\r\n");
                     I2C1_State = 15;            // Only 1 byte to read
                 }
             }
             TimeoutCheck(11);
             break;

         case 12:
             ACK_I2C1_CR1_bit = 1;               // Set ACK bit
             set_timer(I2C1Timer,500);
             DEBUGOUT("Going to state 13\r\n");
             I2C1_State = 13;

         case 13:
             if (RxNE_I2C1_SR1_bit) {            // Wait for data
                 *RdBufPtr++ = I2C1_DR;          // Read a byte of data and put in buffer
                 set_timer(I2C1Timer,500);
                 DEBUGOUT("Going to state 14\r\n");
                 I2C1_State = 14;
             }
             TimeoutCheck(13);
             break;

         case 14:
             RdRegAddr++;                        // Diagnostic only
             RdByteCnt--;                        // Decrement read count
             if (RdByteCnt > 1) {
                 set_timer(I2C1Timer,500);
                 DEBUGOUT("Going to state 13\r\n");
                 I2C1_State = 13;                // Back up one state
             } else {
                 DEBUGOUT("Going to state 15\r\n");
                 I2C1_State = 15;                // Set up for last byte
             }
             break;

         case 15:
             ACK_I2C1_CR1_bit = 0;               // Clear ACK bit
             STOP_I2C1_CR1_bit = 1;              // Set STOP bit
             set_timer(I2C1Timer,500);
             DEBUGOUT("Going to state 16\r\n");
             I2C1_State = 16;
             break;


         case 16:
             if (RxNE_I2C1_SR1_bit) {            // Wait for data
                 *RdBufPtr++ = I2C1_DR;          // Read last byte of data and put in buffer
                 DEBUGOUT("Going to state 17\r\n");
                 I2C1_State = 17;
             }
             TimeoutCheck(16);
             break;

         case 17:
             STOP_I2C1_CR1_bit = 0;              // Clear STOP bit
             free_timer(I2C1Timer);              // Return I2C1 timer to pool
             
             TempPtr[0] = 0xFF;                  // Set buffer valid flag
             DEBUGOUT("Going to state 0\r\n");
             I2C1_State = 0;
             
             // Diagnostic to display the read data buffer just now obtained
             DisplayData(TempPtr,TempCnt);
             
             break;

     /*********************************************************************************************
       Write registers sequence.
       Assumption: user has set up the starting register address - WrRegAddr
                   user has set up the number of registers used - WrByteCnt
                   user has set up a pointer to a buffer - WrBufPtr
     *********************************************************************************************/
     
         case 18:
             I2C1Timer = get_timer();            // Get a timer handle
             set_timer(I2C1Timer,5000);          // Set timer
#if DEBUG
             if (I2C1_SR1 != 0x00) {
                 BufNum = GetBuffer();
                 sprintf(BufNum,"I2C status register: %04X\r\n",I2C1_SR1);
                 UART6_PutQueue(BufNum);

             }
#endif
             
             I2C1_SR1 = 0x0000;
             DEBUGOUT("Going to state 19\r\n");
             I2C1_State = 19;
             break;
     
         case 19:
             if (!BUSY_I2C1_SR2_bit) {           // Make sure bus is not busy
                 START_I2C1_CR1_bit = 1;         // Send Start bit
                 set_timer(I2C1Timer,500);
                 DEBUGOUT("Going to state 20\r\n");
                 I2C1_State = 20;
             }
             TimeoutCheck(19);
             break;

         case 20:
             if (SB_I2C1_SR1_bit) {              // When Start Flag is set
                 I2C1_DR = (STMPE610ADDR);       // Send slave address + write
                 set_timer(I2C1Timer,500);
                 DEBUGOUT("Going to state 21\r\n");
                 I2C1_State = 21;
             }
             TimeoutCheck(20);
             break;

         case 21:
             if (ADDR_I2C1_SR1_bit) {            // Read SR1 waiting for ADDR bit
                 set_timer(I2C1Timer,500);
                 DEBUGOUT("Going to state 22\r\n");
                 I2C1_State = 22;
             }
             TimeoutCheck(21);
             break;

         case 22:
             if (TRA_I2C1_SR2_bit) {             // Read SR2 - this will clear ADDR in SR1
                 set_timer(I2C1Timer,500);
                 DEBUGOUT("Going to state 23\r\n");
                 I2C1_State = 23;
             }
             TimeoutCheck(22);
             break;

         case 23:
             if (TxE_I2C1_SR1_bit) {             // Wait for transmitter empty bit
/*
                 // Diagnostic to display the register address that is being written
                 BufNum = GetBuffer();
                 sprintf(BufNum,"Reg addr %02X\r\n",WrRegAddr);
                 UART6_PutQueue(BufNum);
*/
                 I2C1_DR = WrRegAddr;            // Send the register address that is to be written
                 set_timer(I2C1Timer,500);
                 DEBUGOUT("Going to state 24\r\n");
                 I2C1_State = 24;
             }
             break;

         case 24:
             if (TxE_I2C1_SR1_bit) {             // Wait for transmitter empty bit
                 if (WrByteCnt > 1) {
                     set_timer(I2C1Timer,500);
                     DEBUGOUT("Going to state 25\r\n");
                     I2C1_State = 25;            // More than 1 byte to write
                 } else {
                     DEBUGOUT("Going to state 26\r\n");
                     I2C1_State = 26;            // only 1 byte to write
                 }
             }
             TimeoutCheck(24);
             break;
             
         case 25:
/*
             // Diagnostic to display the data pointed to by the write buffer pointer
             BufNum = GetBuffer();
             sprintf(BufNum,"Data %02X\r\n",*WrBufPtr);
             UART6_PutQueue(BufNum);
*/
             I2C1_DR = *WrBufPtr++;              // Put byte in data register
             WrRegAddr++;                        // Diagnostic only
             WrByteCnt--;                        // Decrement write byte count
             I2C1_State = 24;
             break;

         case 26:
/*
             // Diagnostic to display the data pointed to by the write buffer pointer
             BufNum = GetBuffer();
             sprintf(BufNum,"Data %02X\r\n",*WrBufPtr);
             UART6_PutQueue(BufNum);
*/
             I2C1_DR = *WrBufPtr;                // Put last byte in data register
             set_timer(I2C1Timer,500);
             DEBUGOUT("Going to state 27\r\n");
             I2C1_State = 27;
             break;
             
         case 27:
             if (TxE_I2C1_SR1_bit) {             // Wait for transmitter empty bit
                 DEBUGOUT("Going to state 28\r\n");
                 I2C1_State = 28;
             }
             TimeoutCheck(28);
             break;

         case 28:
             STOP_I2C1_CR1_bit = 1;              // Set STOP bit
             DEBUGOUT("Going to state 29\r\n");
             I2C1_State = 29;
             break;

         case 29:
             free_timer(I2C1Timer);              // Return I2C1 timer to pool
             DEBUGOUT("Going to state 0\r\n");
             I2C1_State = 0;
             break;
     }
}

/*************************************************************************************************/
// EOF
/*************************************************************************************************/