/**************************************************************************************************
  TouchPanel.c
  Driver for the touch panel.
  Notes:
      A fifo stack is used to hold commands for the I2C1 driver to execute. Each command is a
      structure that holds the following values:
        char *Buff;    // Pointer to write/read buffer
        char CmdCode;  // Command: 1 - write, 2 = read
        char RegAd;    // Starting register address
        char Count;    // Number of registers to write/read using STMPE610 auto-increment
      An input pointer and an output pointer keep track of placing and extracting command
      structures to/from the fifo. The fifo is accessed through PutFifo() and GetFifo() functions.
      Place the GetFifo() call in the main loop to check for commands on the fifo often.
      
      A state machine is used to break the many calls to the I2C1 driver so that other tasks
      have a chance to run simultaneously. There are only 2 sequences that the state machine
      executes. The initialization sequence runs only once to set up the STMPE610 registers. The
      interrupt sequence runs each time the touch panel is pressed. The interrupt sequence
      is first invoked when a touch detect interrupt is sensed. After clearing that interrupt,
      the FIFO threshold interrupt runs the state machine again. This FIFO (capital Letters) is
      inside the STMPE610 chip. The FIFO threshold interrupt keeps repeating as long as the
      pressure remains on the touch panel. Only the first threshold interrupt is used to call
      functions in the driver generated by Visual_TFT. When the pressure is released from the
      touch panel, another touch detect interrupt is generated. This one also calls a function
      in the driver module.
      
**************************************************************************************************/

// Include Files
#include "STMPE610.h"
#include "USB_Driver.h"
#include "Buffers.h"
#include "UART6.h"
#include "Common.h"
#include "I2C.h"
#include "Timers.h"
#include "RGB.h"
#include "TP_Test_objects.h"



// Variables
char IntFlag = 0;                      // Flag to indicate an interrupt has occurred
char ActiveFlag = 0;                   // Indicates that a threshold event has occurred

char STMPE_State = 0;                  // Start in state 0
unsigned short int STMPETimer;         // Timer handle for this module

TCommand CmdFifo[CMDFIFOLEN];          // Command fifo space
char CmdFifoIn = 0;                    // Fifo input pointer
char CmdFifoOut = 0;                   // Fifo output pointer

unsigned int   X;                      // Raw X reading from STMPE610
unsigned int   Y;                      // Raw Y reading from STMPE610
unsigned short Z;                      // Raw Z reading from STMPE610

double XNum;                           // Scaled X value
double YNum;                           // Scaled Y value

char BigBuffer[512];                   // Buffer for reading multiple XYZ data from STMPE610 FIFO



extern char I2C1_State;
extern char RdBuffer[32];
extern char WrBuffer[32];
extern char IntFlag;
extern char PenDown;
extern char NextBuf;

extern unsigned int Xcoord, Ycoord;





/**************************************************************************************************
  External interrupt from STMPE610 occurs on pin PA0.
**************************************************************************************************/
void ExternalInterrupt0() iv IVT_INT_EXTI0 ics ICS_AUTO {

     NVIC_IntDisable(IVT_INT_EXTI0);   // Disable EXTI0 interrupt
     PR0_bit = 1;
     IntFlag = 1;
}

/**************************************************************************************************
  External interrupt enable for STMPE610.
**************************************************************************************************/
void ExtIntEnable(void) {

     // Configure PA0 as input with pull-up
     GPIO_Set_Pin_Mode(&GPIOA_BASE, _GPIO_PIN_0, _GPIO_CFG_MODE_INPUT | _GPIO_CFG_PULL_UP);
     // Set EXTI_IMR register
     MR0_bit = 1;                      // Unmask interrupt request from line 0
     // Set EXTI_RTSR register
     TR0_bit = 0;                      // Disable rising trigger
     // Set EXTI_FTSR register
     TR0_EXTI_FTSR_bit = 1;            // Enable falling trigger
     // Set SYSCFGEN bit in RCC_APB2ENR register
     SYSCFGEN_bit = 1;                 // Enable system configuration controller clock
     // Set SYSCFG_EXTICR1
     SYSCFG_EXTICR1 = 0x00000000;      // Connect PA0 to EXTI0

     NVIC_IntEnable(IVT_INT_EXTI0);    // Enable EXTI0 interrupt
}

/**************************************************************************************************
  Put a command in the fifo.
  This function places a command on the fifo.
**************************************************************************************************/
void PutFifo(char *Bfr, char Cmd, char Addr, char Cnt) {
     //char *BufNum;
/*
     // Diagnostic to display command fifo in pointer
     BufNum = GetBuffer();
     sprintf(BufNum,"CmdFifoIn: %02X\r\n",CmdFifoIn);
     UART6_PutQueue(BufNum);
*/
     CmdFifo[CmdFifoIn].Buff = Bfr;
     CmdFifo[CmdFifoIn].CmdCode = Cmd;
     CmdFifo[CmdFifoIn].RegAd = Addr;
     CmdFifo[CmdFifoIn].Count = Cnt;

     if (++CmdFifoIn > (CMDFIFOLEN - 1))
         CmdFifoIn = 0;

}

/**************************************************************************************************
  Get a command from the fifo and send it to I2C1.
  This function obtains a command from the fifo and executes a write or read on I2C1.
**************************************************************************************************/
char GetFifo(void) {
     int i;
     //char *BufNum;

     if (I2C1_State == 0) {                 // Execute if I2C1 state machine is not busy
         if (CmdFifoIn != CmdFifoOut) {     // Execute if a command is in the fifo
/*
             // Diagnostic to display which buffer is being used
             BufNum = GetBuffer();
             sprintf(BufNum,"%d\r\n",NextBuf);
             UART6_PutQueue(BufNum);
*/
             
             // Determine if writing or reading
             switch (CmdFifo[CmdFifoOut].CmdCode) {
                 
                 case 1: // Writing
                     for (i=0;i<CmdFifo[CmdFifoOut].Count;i++) // Transfer data to write buffer
                          WrBuffer[i] = *(CmdFifo[CmdFifoOut].Buff+i);
                     
                     WriteRegs(CmdFifo[CmdFifoOut].RegAd,CmdFifo[CmdFifoOut].Count,CmdFifo[CmdFifoOut].Buff);
/*
                     // Diagnostic to display write buffer contents
                     BufNum = GetBuffer();
                     sprintf(BufNum,"WrBuffer: %02X %02X %02X %02X %02X %02X %02X %02X\r\n",WrBuffer[0],
                     WrBuffer[1],WrBuffer[2],WrBuffer[3],WrBuffer[4],WrBuffer[5],WrBuffer[6],WrBuffer[7]);
                     UART6_PutQueue(BufNum);
*/
                     break;
                 
                 case 2: // Reading
                     if (I2C1_State != 0)
                         UART6_PutQueue("I2C1 state error\r\n");
                     else
                         ReadRegs(CmdFifo[CmdFifoOut].RegAd,CmdFifo[CmdFifoOut].Count,CmdFifo[CmdFifoOut].Buff);
                     break;
                 
                 default:
                     UART6_PutQueue("Unknown command\r\n");
                     break;
             }
             if (++CmdFifoOut > (CMDFIFOLEN - 1)) { // Adjust fifo out pointer
                 CmdFifoOut = 0;
             }
             return 0;                 // Return command sent to I2C1
         } else return 1;              // Return fifo empty status
     } else  return (-1);              // Return I2C1 busy status

}

/**************************************************************************************************
  The timeout subroutine.
  Diagnostic function to reset the state machine if something hangs up.
**************************************************************************************************/
void STMPE_TimeoutCheck(unsigned TimeoutState) {
     char *BufNum;

     if (check_timer(STMPETimer) == 0) {         // If timeout

         BufNum = GetBuffer();
         sprintf(BufNum,"STMPE has timed out in state %d\r\n",TimeoutState);
         UART6_PutQueue(BufNum);
         free_timer(STMPETimer);                 // Release Wi-Fi timer back to the pool
         STMPE_State = 0;                        // Return to holding state
     }
}

/**************************************************************************************************
  STMPE610 state machine.
**************************************************************************************************/
void STMPE_SM(void) {
#if STMPEDEBUG
     static char *BufNum;
#endif
     static char *CmdBufNum;
     
     static char Size;
     static int Number;

     switch (STMPE_State) {
         case 0: // Hold here if the interrupt flag is clear
             if (IntFlag) {            // Interrupt flag?
                 IntFlag = 0;          // Clear the interrupt flag
                 STMPEDEBUGOUT("\nInterrupt\r\n");
                 //RGB_Set_Red(); // Diagnostic
                 STMPE_State = 18;     // Go service the interrupt
             }
             break;

     /*********************************************************************************************
       STMPE610 register initialization sequence.
     *********************************************************************************************/
         case 1: // Get a timer handle and set timeout value
             STMPETimer = get_timer();                // Get a timer handle
             set_timer(STMPETimer,5000);              // Set timer
             STMPE_State = 2;
             break;

         case 2: // Execute a soft reset to the STMPE610
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x02;                 // Soft reset
                 PutFifo(CmdBufNum,WR,SYS_CTRL1,1);   // Reg 0x03 SYS_CTRL1
                 STMPEDEBUGOUT("\nSending reset command\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 3;
             }
             STMPE_TimeoutCheck(2);
             break;
     
         case 3: // Turn on all module clocks
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x00;                 // All clocks on
                 PutFifo(CmdBufNum,WR,SYS_CTRL2,1);   // Reg 0x04 SYS_CTRL2
                 STMPEDEBUGOUT("\nEnabling all clocks\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 4;
             }
             STMPE_TimeoutCheck(3);
             break;
     
         case 4: // Make unused pins into GPIO ports
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x0C;                 // GPIO 2 and 3 are alternate function
                 PutFifo(CmdBufNum,WR,GPIO_AF,1);     // Reg 0x17 GPIO_AF
                 STMPEDEBUGOUT("\nSetting GPIO alt function for bits 2 and 3\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 5;
             }
             STMPE_TimeoutCheck(4);
             break;

         case 5: // Set unused GPIO ports to outputs
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x0C;                 // GPIO 2 and 3 are outputs
                 PutFifo(CmdBufNum,WR,GPIO_DIR,1);    // Reg 0x13 GPIO_DIR
                 STMPEDEBUGOUT("\nSeting GPIO bits 2 and 3 for output\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 6;
             }
             STMPE_TimeoutCheck(5);
             break;

         case 6: // Set up ADC
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x48;                 // Sample time  to 80, 12-bit ADC, internal reference
                 PutFifo(CmdBufNum,WR,ADC_CTRL1,1);   // Reg 0x20 ADC_CTRL1
                 STMPEDEBUGOUT("\nSeting sampling time, 12-bit ADC, internal reference\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 7;
             }
             STMPE_TimeoutCheck(6);
             break;

         case 7: // Set ADC frequency
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x01;                 // ADC frequency to 3.25 MHz
                 PutFifo(CmdBufNum,WR,ADC_CTRL2,1);   // Reg 0x21 ADC_CTRL2
                 STMPEDEBUGOUT("\nSeting ADC frequency\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 8;
             }
             STMPE_TimeoutCheck(7);
             break;

         case 8: // Set up touch delays
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 //CmdBufNum[0] = 0x9A;                 // Average 4 samples, 500 us touch delay and settling time (doesn't work)
                 //CmdBufNum[0] = 0xA3;                 // Average 4 samples, 1 ms touch delay 1 ms settling time (doesn't work)
                 CmdBufNum[0] = 0xB5;                 // Average 4 samples, 10 ms touch delay 10 ms settling time
                 //CmdBufNum[0] = 0xBE;                 // Average 4 samples, 50 ms touch delay 50 ms settling time
                 //CmdBufNum[0] = 0xFF;                 // Average 8 samples, 50 ms touch delay 100 ms settling time

                 PutFifo(CmdBufNum,WR,TSC_CFG,1);     // Reg 0x41 TSC_CFG
                 STMPEDEBUGOUT("\nSeting delay and settling time\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 9;
             }
             STMPE_TimeoutCheck(8);
             break;

         case 9: // Only 1 point FIFO mode
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x01;                 // Single point reading method
                 PutFifo(CmdBufNum,WR,FIFO_TH,1);     // Reg 0x4A FIFO_TH
                 STMPEDEBUGOUT("\nSeting FIFO threshold for single point reading\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 10;
             }
             STMPE_TimeoutCheck(9);
             break;

         case 10: // Reset FIFO to clear it
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x01;                 // FIFO reset
                 PutFifo(CmdBufNum,WR,FIFO_STA,1);    // Reg 0x4B FIFO_STA
                 STMPEDEBUGOUT("\nResetting FIFO\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 11;
             }
             STMPE_TimeoutCheck(10);
             break;

         case 11: // More setups
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x07;                 // Fractional part to 7, whole part to 1
                 PutFifo(CmdBufNum,WR,TSC_FRACTION_Z,1); // Reg 0x56 TSC_FRACTION_Z
                 STMPEDEBUGOUT("\nSetting TSC fraction to 7\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 12;
             }
             STMPE_TimeoutCheck(11);
             break;

         case 12: // Set maximum drive current
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 //CmdBufNum[0] = 0x01;                 // I drive for 50 mA max
                 CmdBufNum[0] = 0x00;                 // I drive for 20 mA max
                 PutFifo(CmdBufNum,WR,TSC_I_DRIVE,1); // Reg 0x58 TSC_I_DRIVE
                 STMPEDEBUGOUT("\nSetting TSC I drive\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 13;
             }
             STMPE_TimeoutCheck(12);
             break;

         case 13: // Enable TSC. This also clears FIFO reset
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x01;                 // Enable TSC
                 PutFifo(CmdBufNum,WR,TSC_CTRL,1);    // Reg 0x40 TSC_CTRL
                 STMPEDEBUGOUT("\nEnabling TSC\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 14;
             }
             STMPE_TimeoutCheck(13);
             break;

         case 14: // Set FIFO threshold interrupt enable and touch detect interrupt enable
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x03;                 // Enable interrupts
                 PutFifo(CmdBufNum,WR,INT_EN,1);      // Reg 0x0A INT_EN
                 STMPEDEBUGOUT("\nEnabling FIFO threshold and touch detect interrupts\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 15;
             }
             STMPE_TimeoutCheck(14);
             break;

         case 15: // Clear all interrupts
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0xFF;                 // Clear all interrupts
                 PutFifo(CmdBufNum,WR,INT_STA,1);     // Reg 0x0B INT_STA
                 STMPEDEBUGOUT("\nClearing all interrupts\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 16;
             }
             STMPE_TimeoutCheck(15);
             break;
         
         case 16: // Set up external interrupt level and global enable
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x03;                 // Set edge type and global int enable bits
                 PutFifo(CmdBufNum,WR,INT_CTRL,1);    // Reg 0x09 INT_CTRL
                 STMPEDEBUGOUT("\nSetting global interrupt enable\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 17;
             }
             STMPE_TimeoutCheck(16);
             break;

         case 17: // Free timer and return to holding state
             if (I2C1_State == 0) {
                 STMPEDEBUGOUT("\nRegister setup complete\r\n");
                 free_timer(STMPETimer);
                 STMPE_State = 0;
                 
                 //RGB_Clr_Green();  // Diagnostic
             }
             STMPE_TimeoutCheck(17);
             break;
     
     /*********************************************************************************************
       STMPE610 interrupt processing and reading a single value sequence.
     *********************************************************************************************/
         case 18:
             STMPETimer = get_timer();                // Get a timer handle
             set_timer(STMPETimer,5000);              // Set timer
/*
             // Diagnostic to display the timer handle
             BufNum = GetBuffer();
             sprintf(BufNum,"Timer number: %04X\r\n",STMPETimer);
             UART6_PutQueue(BufNum);
*/
             STMPE_State = 19;
             break;
         
         case 19: // Read the interrupt status register
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 memset(CmdBufNum,0,8);               // Clear data buffer
                 PutFifo(CmdBufNum,RD,INT_STA,1);     // Reading register 0x0B (INT_STA)
                 STMPEDEBUGOUT("\nReading interrupt status register\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 20; //20
             }
             STMPE_TimeoutCheck(19);
             break;
     
         case 20: // Check interrupts
             if ((CmdBufNum[0] == 0xFF) && (I2C1_State == 0)) { // Data available?

                 if ((CmdBufNum[1] & 0x02) == 0x02) { // Threshold interrupt?
                     STMPEDEBUGOUT("\nFIFO threshold interrupt\r\n");
                     set_timer(STMPETimer,5000);      // Set timer
                     STMPE_State = 21;
                 } else if ((CmdBufNum[1] & 0x01) == 0x01) { // Touch detect interrupt?
                     
                     if (PenDown == 0) {
                         PenDown = 1;
                         //RGB_Set_Blue(); // Diagnostic
                     } else if (PenDown == 1) {
                         PenDown = 0;
                         //RGB_Clr_Blue(); // Diagnostic
                         Process_TP_Up(Xcoord,Ycoord);
                     }
                     STMPEDEBUGOUT("\nTouch detect interrupt\r\n");
                     set_timer(STMPETimer,5000);      // Set timer
                     STMPE_State = 28;
                 } else {
                     STMPEDEBUGOUT("\nShouldn't be here\r\n");
                     set_timer(STMPETimer,5000);      // Set timer
                     STMPE_State = 28;
                 }
             }
             STMPE_TimeoutCheck(20);
             break;
     
         // Here after threshold interrupt
         case 21: // Get number of points available in FIFO
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 memset(CmdBufNum,0,8);               // Clear data buffer
                 PutFifo(CmdBufNum,RD,FIFO_SIZE,1);   // Reading register 0x4C (FIFO_SIZE)
                 STMPEDEBUGOUT("\nReading number of points\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 22;
             }
             STMPE_TimeoutCheck(21);
             break;

         case 22: // Wait to receive number of points data
             if ((CmdBufNum[0] == 0xFF) && (I2C1_State == 0)) { // Data available?

                 Size = CmdBufNum[1];
                 Number = Size*4;                 // 4 bytes of data for each point
                 STMPE_State = 23;
#if STMPEDEBUG
                 // Diagnostic
                 BufNum = GetBuffer();
                 sprintf(BufNum,"\nSize: %d  Number: %d\r\n",Size,Number);
                 UART6_PutQueue(BufNum);
#endif
             }
             STMPE_TimeoutCheck(22);
             break;

         case 23: // Read data from FIFO according to number of points
             if (I2C1_State == 0) {
                 //CmdBufNum = GetCmdBuf();
                 CmdBufNum = &BigBuffer;              // Use bigger buffer just in case
                 memset(CmdBufNum,0,8);               // Clear data buffer
                 PutFifo(CmdBufNum,RD,TSC_DATA_1,Number); // Reading FOFO data 0xD7 (TSC_DATA_1)
                 STMPEDEBUGOUT("\nReading data\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 24;
             }
             STMPE_TimeoutCheck(23);
             break;

         case 24: // Wait for data then extract X, Y, and Z values
             if ((CmdBufNum[0] == 0xFF) && (I2C1_State == 0) && (Size > 1)) {
                 STMPEDEBUGOUT("\nToo many points in FIFO\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 25;                    // Don't process data
             
             } else if ((CmdBufNum[0] == 0xFF) && (I2C1_State == 0) && (Size == 1)) {
                 X = (CmdBufNum[1] << 4) | ((CmdBufNum[2] & 0xF0) >> 4);
                 Y = ((CmdBufNum[2] & 0x0F) << 8) | CmdBufNum[3];
                 Z = CmdBufNum[4];

                 // Using a display with 480 X 272 pixels
                 XNum = X / 8.6; YNum = Y / 14.8;     // Scale touch panel to display pixels
                 
                 Xcoord = (unsigned)XNum;
                 Ycoord = (unsigned)YNum;
#if STMPEDEBUG
                 // Diagnostic
                 BufNum = GetBuffer();
                 sprintf(BufNum,"\nX: %04d  Y: %04d\r\n",Xcoord,Ycoord);
                 UART6_PutQueue(BufNum);
#endif
                 // Execute following on fist pass only, ignore subsequently
                 if (ActiveFlag == 0) {
                     ActiveFlag = 1;
                     Process_TP_Press(Xcoord,Ycoord);
                     Process_TP_Down(Xcoord,Ycoord);
                 }
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 25;
             }
             STMPE_TimeoutCheck(24);
             break;

         // Here after reading FIFO data
         case 25: // Reset FIFO
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x01;                 // FIFO reset
                 PutFifo(CmdBufNum,WR,FIFO_STA,1);    // Reg 0x4B FIFO_STA
                 STMPEDEBUGOUT("\nResetting FIFO\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 26;
             }
             STMPE_TimeoutCheck(25);
             break;
             
         case 26: // Enable TSC. This also clears FIFO reset
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x01;                 // Enable TSC
                 PutFifo(CmdBufNum,WR,TSC_CTRL,1);    // Reg 0x40 TSC_CTRL
                 STMPEDEBUGOUT("\nEnabling TSC\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 27;
             }
             STMPE_TimeoutCheck(26);
             break;
         
         case 27: // Clear interrupts
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x02;                 // Clear FIFO threshold interrupt
                 PutFifo(CmdBufNum,WR,INT_STA,1);     // Writing register 0x0B (INT_STA)
                 STMPEDEBUGOUT("\nClearing FIFO threshold interrupt\r\n");
                 set_timer(STMPETimer,50);            // Set timer
                 STMPE_State = 34;
             }
             STMPE_TimeoutCheck(27);
             break;

         // Here after touch detect interrupt
         case 28: // Get number of points available in FIFO
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 memset(CmdBufNum,0,8);               // Clear data buffer
                 PutFifo(CmdBufNum,RD,FIFO_SIZE,1);   // Reading register 0x4C (FIFO_SIZE)
                 STMPEDEBUGOUT("\nReading number of points\r\n");
                 set_timer(STMPETimer,100);          // Set timer
                 STMPE_State = 29;
             }
             STMPE_TimeoutCheck(28);
             break;
         
         case 29: // Wait to receive number of points data
             if ((CmdBufNum[0] == 0xFF) && (I2C1_State == 0)) {
#if STMPEDEBUG
                 // Diagnostic
                 BufNum = GetBuffer();
                 sprintf(BufNum,"\nSize: %d\r\n",CmdBufNum[1]);
                 UART6_PutQueue(BufNum);
#endif
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 30;
             }
             STMPE_TimeoutCheck(29);
             break;

         case 30: // On Button up interrupt, clear first time flag
             if (ActiveFlag == 1)
                 ActiveFlag = 0;
                 
             set_timer(STMPETimer,5000);              // Set timer
             STMPE_State = 31;
             break;

         case 31: // Reset FIFO
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x01;                 // FIFO reset
                 PutFifo(CmdBufNum,WR,FIFO_STA,1);    // Reg 0x4B FIFO_STA
                 STMPEDEBUGOUT("\nResetting FIFO\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 32;
             }
             STMPE_TimeoutCheck(31);
             break;
         
         case 32: // Enable TSC. This also clears FIFO reset
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x01;                 // Enable TSC
                 PutFifo(CmdBufNum,WR,TSC_CTRL,1);    // Reg 0x40 TSC_CTRL
                 STMPEDEBUGOUT("\nEnabling TSC\r\n");
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 33;
             }
             STMPE_TimeoutCheck(32);
             break;
         
         case 33: // Clear interrupts
             if (I2C1_State == 0) {
                 CmdBufNum = GetCmdBuf();
                 CmdBufNum[0] = 0x01;                 // Clear touch detect interrupt
                 PutFifo(CmdBufNum,WR,INT_STA,1);     // Writing register 0x0B (INT_STA)
                 STMPEDEBUGOUT("\nClearing touch detect interrupt\r\n");
                 set_timer(STMPETimer,50);            // Set timer
                 STMPE_State = 34;
             }
             STMPE_TimeoutCheck(33);
             break;

     /********************************************************************************************/
         
         case 34: // Small delay after clearing interrupts
             if (check_timer(STMPETimer) == 0) {
                 set_timer(STMPETimer,5000);          // Set timer
                 STMPE_State = 35;
             }
             break;

         case 35: // Free timer and go back to start
             if (I2C1_State == 0) {
                 STMPEDEBUGOUT("\nInterrupt processing complete\r\n");
                 free_timer(STMPETimer);
                 STMPE_State = 0;
                 //RGB_Clr_Red(); // Diagnostic
                 NVIC_IntEnable(IVT_INT_EXTI0);  // Enable EXTI0 interrupt
             }
             STMPE_TimeoutCheck(35);
             break;
     }
}

/**************************************************************************************************
  EOF
**************************************************************************************************/