/**************************************************************************************************
  UART6.c
  This UART module communicates with the FTDI Serial-to-USB FT232RL chip on the MikroMedia+ for
  STM32 shield. It's mainly used for diagnostic messages and requests for information.
**************************************************************************************************/

// Include Files
#include "UART6.h"
#include "Buffers.h"


#pragma funcall UART6_Rx1, UART6_Rx2

// Variables
char RxIn;                   // Input pointer to the next byte in receive FIFO
char RxOut;                  // Output pointer to the next byte in receive FIFO
char RxQueue[rxqsize];       // Re-circulating receive FIFO

char RcvBuf[RcvBufSize];     // Receive messages are assembled in this buffer
char RcvBufPtr;              // Pointer to the next free byte in RcvBuf


char TxIn;                   // Input pointer next element in the transmit FIFO
char TxOut;                  // Output pointer to the next element in the transmit FIFO
char *TxQueue[txqsize];      // Queue of message pointers in the transmit FIFO

char *XmtBuf;                // Pointer to the current message being transmitted
char XmtBufPtr;              // Points to a byte in transmit buffer


// The receive message routine has 2 states
tPointerFcn UART6_RxFuncPtr = UART6_Rx1;  // Start in State 1



/*************************************************************************************************/
// Hardware Level Functions
/**************************************************************************************************
  Interrupt Handler */
void UART6_Interrupt() iv IVT_INT_USART6 ics ICS_AUTO {
     char R_data;        // Receive data byte
     char T_data;        // Transmit data byte


     if ((RXNE_USART6_SR_bit) /* || (ORE_bit) */ ) {
         R_data = USART6_DR;           // Fetch a byte from receive register
         RxQueue[RxIn++] = R_data;     // Put data byte in re-circulating buffer, increment pointer
         if (RxIn > (rxqsize-1))       // Adjust pointer if necessary
             RxIn = 0;
     }

     if ((TXE_USART6_SR_bit)&&(TXEIE_USART6_CR1_bit)) {
         T_data = XmtBuf[XmtBufPtr++]; // Get a byte from the transmit buffer
         if (T_data == NUL) {          // If it is the end of message marker
             TXEIE_USART6_CR1_bit = 0; // Disable transmitter interrupts
         } else {
             USART6_DR = T_data;       // Transfer data byte to transmit register
         }
     }
}

/**************************************************************************************************
  This routine sets the baud rate. */
void UART6_SetBaud(void) {
     UART6_Init_Advanced(9600,_UART_8_BIT_DATA,_UART_NOPARITY,_UART_ONE_STOPBIT,&_GPIO_MODULE_USART6_PC67);
     NVIC_IntEnable(IVT_INT_USART6);   // Enable interrupt vector
}

/**************************************************************************************************
  This routine starts the receiver. */
void UART6_StartRx(void) {
     RXNEIE_USART6_CR1_bit = 1;        // Interrupt when receive buffer is full (RXNE=1)
}

/**************************************************************************************************
  This routine starts the transmitter. */
void UART6_StartTx(void) {
     TXEIE_USART6_CR1_bit = 1;         // Interrupt when transmit buffer is empty (TXE=1)
}
//-------------------------------------------------------------------------------------------------



/*************************************************************************************************/
// UART6 Functions
/**************************************************************************************************
  Initialize UART */
void UART6_Initialize(void) {
     UART6_RxQueueInit();
     UART6_TxQueueInit();

     UART6_SetBaud();
     UART6_StartRx();
}

// The receive portion ****************************************************************************
/**************************************************************************************************
  This routine initializes the receive buffer in and out pointers. */
void UART6_RxQueueInit(void) {
     RxIn = RxOut = 0;                 // Set pointers to 0
     RcvBufPtr = 0;
}

/**************************************************************************************************
  This routine retrieves a byte from the re-circulating buffer if present. Returns 0 if no data
  available. Returns 1 if a data byte has been transferred successfully. The pointer to the data
  byte is passed to the routine. */
char UART6_GetChar(char *rxbyte) {
     if (RxIn != RxOut) {              // If there is a character in the buffer
        *rxbyte = RxQueue[RxOut++];    // Extract the character and update the pointer
        if (RxOut > (rxqsize-1))       // Check for wrap
           RxOut = 0;
        return 1;
     }else{
        return 0;                      // No data available
     }
}

/**************************************************************************************************
  UART_Rx State Machine. Call this function in the main loop. */
void UART6_Rx(void) {
     UART6_RxFuncPtr();    // Use the function pointer to go to a state
}

/**************************************************************************************************
  UART_Rx State 1: Accumulate received characters in a buffer until a Line Feed character is
  received then insert a NUL character into the buffer and go to State 2. */
void UART6_Rx1(void){
     char rxchar;

     if (UART6_GetChar(&rxchar)) {
         switch (rxchar) {
             case NUL:         // Discard NUL character
                 break;

             case '\r':        // Discard return character
                 break;

             case '\n':
                 RcvBuf[RcvBufPtr++] = rxchar;       // Put received character into buffer
                 RcvBuf[RcvBufPtr] = 0;              // End marker
                 RcvBufPtr = 0;                      // Point to start of buffer
                 UART6_RxFuncPtr = UART6_Rx2;        // Go to state 2
                 break;

             default:
                 if (RcvBufPtr < (RcvBufSize-2))     // Prevents buffer overrun
                     RcvBuf[RcvBufPtr++] = rxchar;   // Put character in buffer
                 break;
         }
     }
}

/**************************************************************************************************
  UART_Rx State 2: Execute the command indicated by the first byte in the buffer then return to
  State 1. */
void UART6_Rx2(void) {
     char *BufNum;

     switch (RcvBuf[0]) {

         case 'a': // Diagnostic command to see if serial communications is working OK
             UART6_PutQueue("Command a received\r\n");
             break;

         // Add more commands to suit application
         
         
         
         case 'z': // Do a software reset
             SystemReset();
             break;


         default:  // Catch any unimplemented commands
             UART6_PutQueue("Unknown command: ");

             BufNum = GetBuffer();               // Get a free buffer
             (void)sprintf(BufNum,"%c\r\n",RcvBuf[0]);
             UART6_PutQueue(BufNum);             // Send the message
             break;
     }
     UART6_RxFuncPtr = UART6_Rx1;                // Go back to state 1
}



// The transmit portion ***************************************************************************
/**************************************************************************************************
  This routine initializes the transmit buffer queue in and out pointers. */
void UART6_TxQueueInit(void) {
     TxIn = TxOut = 0;         // Set pointers to 0
     XmtBufPtr = 0;
}

/**************************************************************************************************
  This routine makes an entry into the transmit queue of pointers and updates the in pointer to
  the next available location. */
void UART6_PutQueue(char *msg) {
     TxQueue[TxIn++] = msg;    // Put the message pointer into FIFO
     if (TxIn > (txqsize-1))   // Adjust FIFO in pointer
         TxIn = 0;
}

/**************************************************************************************************
  This routine enters a pointer to a text message and starts the transmitter. */
void UART6_SendMsg(char *s) {
     XmtBuf = s;               // Pointer to the message for the interrupt handler
     XmtBufPtr = 0;            // Set character counter to start of text message
     UART6_StartTx();          // Turn on the transmitter
}

/**************************************************************************************************
  This routine checks the transmit buffer queue and if it is not empty, it extracts the pointer
  to the message and sets up the transmitter. Call this function in the main loop. */
void UART6_CheckQueue(void) {
     if (TXEIE_USART6_CR1_bit == 0) {   // Don't try to start transmitter if already transmitting
         if (TxIn != TxOut) {                 // If pointers are not the same
             UART6_SendMsg(TxQueue[TxOut++]); // Hand off the message pointer and increment counter
             if (TxOut > (txqsize-1))         // Check for wrap
                 TxOut = 0;
         }
     }
}